#!/bin/sh
set -e
# Install SBCL
#
# Installation is done under $INSTALL_ROOT, defaulting to /usr/local
#
# Suffix defaults to output from (LISP-IMPLEMENTATION-VERSION),
# but INSTALL_SUFFIX can be used to override it.
#
# Runtime and symlink:
#  bin/sbcl-<version>
#  bin/sbcl             -> sbcl-<version>
#
# Core and contribs (and symlink):
#  lib/sbcl             -> sbcl-<version>
#  lib/sbcl-<version>/sbcl.core
#  lib/sbcl-<version>/<each contrib>/<files>
#  lib/sbcl-<version>/site-systems/

# Get the right defaults for Windows / Unix.
if [ "$OSTYPE" = "cygwin" -o "$OSTYPE" = "msys" ] ; then
    default_install_root="$PROGRAMFILES/sbcl"
    exe=.exe
else
    default_install_root=/usr/local
    exe=""
fi

# Before doing anything else, make sure we have an SBCL to install
if test -f "src/runtime/sbcl$exe"
then
    if test -f output/sbcl.core
    then
        true
    else
        echo "output/sbcl.core not found, aborting installation."
        exit 1
    fi
else
    echo "src/runtime/sbcl$exe not found, aborting installation."
    exit 1
fi

# And that we have a make..
. ./find-gnumake.sh
find_gnumake

. ./sbcl-pwd.sh
sbcl_pwd

SBCL="$SBCL_PWD/src/runtime/sbcl --noinform --core $SBCL_PWD/output/sbcl.core --noinform --noprint --no-userinit --no-sysinit --disable-debugger"

default_install_suffix=`$SBCL \
                        --eval '(format t "-~A" (lisp-implementation-version))' \
                        --eval '(quit)'`

build_root="${BUILD_ROOT:-}"
install_root="${INSTALL_ROOT:-$default_install_root}"
install_suffix="${INSTALL_SUFFIX:-$default_install_suffix}"

man_dir="${MAN_DIR:-$install_root/share/man}"
info_dir="${INFO_DIR:-$install_root/share/info}"
doc_dir="${DOC_DIR:-$install_root/share/doc/sbcl}"

SBCL_HOME="$install_root/lib/sbcl$install_suffix"

# Build the directory hierarchy.
mkdir -p "$build_root$install_root/bin"
mkdir -p "$build_root$install_root/lib/sbcl$install_suffix/site-systems"
mkdir -p "$build_root$man_dir/man1"
mkdir -p "$build_root$info_dir"
mkdir -p "$build_root$doc_dir/html"
mkdir -p "$build_root$doc_dir/html"

# If there is an old-style SBCL installed (not a symlink to suffixed
# version), rename it to sbcl.old -- let's just hope no-one needs to require
# any new contribs...
if test -f "$build_root$install_root/bin/sbcl$exe"
then
    mv "$build_root$install_root/bin/sbcl$exe" \
	"$build_root$install_root/bin/sbcl.old$exe"
fi
if test -d "$build_root$install_root/lib/sbcl"
then
    mv "$build_root$install_root/lib/sbcl" "$build_root$install_root/lib/sbcl.old" 
fi

# Install core and runtime
cp "src/runtime/sbcl$exe" "$build_root$install_root/bin/sbcl$install_suffix$exe"
cp "output/sbcl.core" "$build_root$install_root/lib/sbcl$install_suffix/sbcl.core"

# Install contribs
SBCL_BUILDING_CONTRIB=1
export SBCL SBCL_BUILDING_CONTRIB

for i in contrib/*; do
    test -d $i && test -f $i/test-passed || continue;
    INSTALL_DIR="$SBCL_HOME"/`basename $i `
    export INSTALL_DIR
    mkdir -p "$build_root$INSTALL_DIR" && $GNUMAKE -C $i install
done

# Symlink bin/sbcl and lib/sbcl/
ln -sf "$build_root$install_root/bin/sbcl$install_suffix$exe" \
    "$build_root$install_root/bin/sbcl$exe"
ln -sf "$build_root$install_root/lib/sbcl$install_suffix" \
    "$build_root$install_root/lib/sbcl"

echo
echo "SBCL has been installed:"
echo " binary $build_root$install_root/bin/sbcl$install_suffix$exe"
echo "   (symlinked as $build_root$install_root/bin/sbcl$exe)"
echo " core and contribs in $build_root$install_root/lib/sbcl$install_suffix"
echo "   (symlinked as $build_root$install_root/lib/sbcl)"

# Installing manual & misc bits of documentation
#
# Locations based on FHS 2.3.
# See: <http://www.pathname.com/fhs/pub/fhs-2.3.html>
#
# share/       architecture independent read-only things
# share/man/   manpages, should be the same as man/
# share/info/  info files
# share/doc/   misc documentation

echo
echo "Documentation:"

# man
cp doc/sbcl.1 "$build_root$man_dir/man1/" && echo " man $build_root$man_dir/man1/sbcl.1"

# info
for info in doc/manual/*.info
do
  cp $info "$build_root$info_dir"/ \
      && echo -n " info $build_root$info_dir/`basename $info` $build_root$info_dir/dir" \
      && ( install-info --info-dir="$build_root$info_dir" \
        "$build_root$info_dir"/`basename $info` > /dev/null 2>&1 \
           || echo -n " (could not add to system catalog)" ) \
      && echo
done

for info in doc/manual/*.info-*
do
  cp $info "$build_root$info_dir"/ \
      && echo " info $build_root$info_dir/`basename $info`"
done

# pdf
for pdf in doc/manual/*.pdf
do
  cp $pdf "$build_root$doc_dir"/ \
      && echo " pdf $build_root$doc_dir/`basename $pdf`"
done

# html
for html in doc/manual/sbcl doc/manual/asdf
do
  test -d $html && cp -R -L $html "$build_root$doc_dir"/html \
      && echo " html $build_root$doc_dir/html/`basename $html`/index.html"
done

texts=""
for f in BUGS CREDITS COPYING NEWS
do
    if cp $f "$build_root$doc_dir"/
    then
	texts="$texts, $f"
    fi
done
if test $texts
then
    echo "$texts $build_root$doc_dir/"
fi
